# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created: Mon May 31 16:40:59 2010
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(437, 205)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.l_avail = QtGui.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        font.setPointSize(24)
        self.l_avail.setFont(font)
        self.l_avail.setAlignment(QtCore.Qt.AlignCenter)
        self.l_avail.setObjectName("l_avail")
        self.gridLayout.addWidget(self.l_avail, 0, 0, 1, 5)
        self.l_last = QtGui.QLabel(self.centralwidget)
        self.l_last.setAlignment(QtCore.Qt.AlignCenter)
        self.l_last.setObjectName("l_last")
        self.gridLayout.addWidget(self.l_last, 3, 0, 1, 5)
        self.l_solved_1 = QtGui.QLabel(self.centralwidget)
        self.l_solved_1.setObjectName("l_solved_1")
        self.gridLayout.addWidget(self.l_solved_1, 4, 0, 1, 1)
        self.l_solved_2 = QtGui.QLabel(self.centralwidget)
        self.l_solved_2.setObjectName("l_solved_2")
        self.gridLayout.addWidget(self.l_solved_2, 4, 1, 1, 1)
        self.l_solved_3 = QtGui.QLabel(self.centralwidget)
        self.l_solved_3.setObjectName("l_solved_3")
        self.gridLayout.addWidget(self.l_solved_3, 4, 2, 1, 1)
        self.l_solved_4 = QtGui.QLabel(self.centralwidget)
        self.l_solved_4.setObjectName("l_solved_4")
        self.gridLayout.addWidget(self.l_solved_4, 4, 3, 1, 1)
        self.l_solved_5 = QtGui.QLabel(self.centralwidget)
        self.l_solved_5.setObjectName("l_solved_5")
        self.gridLayout.addWidget(self.l_solved_5, 4, 4, 1, 1)
        self.l_typed = QtGui.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        font.setPointSize(24)
        self.l_typed.setFont(font)
        self.l_typed.setAlignment(QtCore.Qt.AlignCenter)
        self.l_typed.setObjectName("l_typed")
        self.gridLayout.addWidget(self.l_typed, 1, 0, 1, 5)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 437, 24))
        self.menubar.setObjectName("menubar")
        self.menuGame = QtGui.QMenu(self.menubar)
        self.menuGame.setObjectName("menuGame")
        self.menuHelp = QtGui.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionAbout = QtGui.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionNew = QtGui.QAction(MainWindow)
        self.actionNew.setObjectName("actionNew")
        self.actionSolve = QtGui.QAction(MainWindow)
        self.actionSolve.setObjectName("actionSolve")
        self.actionQuit = QtGui.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionWordlists = QtGui.QAction(MainWindow)
        self.actionWordlists.setObjectName("actionWordlists")
        self.menuGame.addAction(self.actionNew)
        self.menuGame.addAction(self.actionSolve)
        self.menuGame.addSeparator()
        self.menuGame.addAction(self.actionWordlists)
        self.menuGame.addSeparator()
        self.menuGame.addAction(self.actionQuit)
        self.menuGame.addSeparator()
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuGame.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QObject.connect(self.actionQuit, QtCore.SIGNAL("activated()"), MainWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "Pynagram", None, QtGui.QApplication.UnicodeUTF8))
        self.l_avail.setText(QtGui.QApplication.translate("MainWindow", "ABCDEFG", None, QtGui.QApplication.UnicodeUTF8))
        self.l_last.setText(QtGui.QApplication.translate("MainWindow", "Last Word:", None, QtGui.QApplication.UnicodeUTF8))
        self.l_solved_1.setText(QtGui.QApplication.translate("MainWindow", "Something", None, QtGui.QApplication.UnicodeUTF8))
        self.l_solved_2.setText(QtGui.QApplication.translate("MainWindow", "Something", None, QtGui.QApplication.UnicodeUTF8))
        self.l_solved_3.setText(QtGui.QApplication.translate("MainWindow", "Something", None, QtGui.QApplication.UnicodeUTF8))
        self.l_solved_4.setText(QtGui.QApplication.translate("MainWindow", "Something", None, QtGui.QApplication.UnicodeUTF8))
        self.l_solved_5.setText(QtGui.QApplication.translate("MainWindow", "Something", None, QtGui.QApplication.UnicodeUTF8))
        self.l_typed.setText(QtGui.QApplication.translate("MainWindow", "TUVWXYZ", None, QtGui.QApplication.UnicodeUTF8))
        self.menuGame.setTitle(QtGui.QApplication.translate("MainWindow", "&Game", None, QtGui.QApplication.UnicodeUTF8))
        self.menuHelp.setTitle(QtGui.QApplication.translate("MainWindow", "&Help", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAbout.setText(QtGui.QApplication.translate("MainWindow", "About", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew.setText(QtGui.QApplication.translate("MainWindow", "New", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+N", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSolve.setText(QtGui.QApplication.translate("MainWindow", "Solve", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSolve.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+L", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setText(QtGui.QApplication.translate("MainWindow", "Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+Q", None, QtGui.QApplication.UnicodeUTF8))
        self.actionWordlists.setText(QtGui.QApplication.translate("MainWindow", "Word Lists", None, QtGui.QApplication.UnicodeUTF8))
        self.actionWordlists.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+O", None, QtGui.QApplication.UnicodeUTF8))




#! /usr/bin/python
#
#     config.py
# 
#     Copyright (c) 2010 Umang Varma <umang.me@gmail.com>.
# 
#     This file is part of Pynagram
# 
#     Pynagram is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     Pynagram is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with Pynagram. If not, see <http://www.gnu.org/licenses/>.
# 
import sys
import os.path
from os import environ
try:
    import json
    json.JSONDecodeError = ValueError
except ImportError:
    import simplejson as json
class pynagram_config(dict):
    """Handles all configuration actions: read, write, defaults, etc"""

    def  __init__(self):
        dict.__init__(self)
        self.changed = False
    
    def _raw_config(self):
        """Returns raw config"""
        try:
            fsock = open(self.pathtoconfigfile())
        except IOError:
            self.writedefault()
            fsock = open(self.pathtoconfigfile())
        configdata = fsock.read()
        fsock.close()
        return configdata

    def readconfig(self):
        """Reads configuration file and returns a dict with data"""
        try:
            data = json.loads(self._raw_config())
        except json.JSONDecodeError:
            self.writedefault()
            data = json.loads(self._raw_config())
        self.clear()
        self.update(data)
        self.setdefault("wordlist", 0)
        if not self["wordlists"]:
            self.adddefaultwl()
            self.writeconfig()

    def writeconfig(self):
        """Writes configuration data provided by the data dict"""
        try:
            fsock = open(self.pathtoconfigfile(), "w")
        except IOError:
            self.createdir()
            fsock = open(self.pathtoconfigfile(), "w")
        self["wordlists"] = [os.path.normpath(x) for x in self["wordlists"]]
        fsock.write(json.dumps(self, indent=4))
        fsock.close()

    def getdefaultwl(self):
        """Returns the default wordlist"""
        tries = [[sys.path[0] + "/../wordlist/wordlist.txt", sys.path[0] + \
                "/wordlist/wordlist.txt", sys.prefix + \
                "/share/pynagram/wordlist.txt"]]
        # A list of list of possible wordlists. Each nested list contains
        # similar wordlists, only one of which will be used. If a wordlist is
        # different, it needs to be placed in a separate list. e.g :
        # [['a1', 'a2', 'a3'], ['b'], ['c']] when a1, a2, a3 are similar
        # wordlists found in different locations and b, c are distinct
        wordlists = []
        for source in tries:
            for path in source:
                if os.path.exists(path):
                    wordlists.append(path)
                    break
        return wordlists
        raise IOError("No wordlist found in any of the expected locations")

    def adddefaultwl(self):
        """Adds the default wordlists"""
        self.setdefault("wordlists", []).extend(self.getdefaultwl())

    def writedefault(self):
        """Writes default config file"""
        self.clear()
        self.adddefaultwl()
        self.writeconfig()

    def pathtoconfig(self):
        """Returns path to configuration file"""
        if sys.platform == 'win32':
             appdata = os.path.join(environ['APPDATA'], "Pynagram")
        else:
             appdata = os.path.expanduser(os.path.join("~", ".pynagram/"))
        return appdata

    def pathtoconfigfile(self):
        """Returns path to config file"""
        return os.path.join(self.pathtoconfig(), "config.txt")

    def createdir(self):
        """Creates the config directory if it doesn't already exist"""
        try:
            os.mkdir(self.pathtoconfig())
        except OSError:
            print self.pathtoconfig() + " already exists. Writing defaults"
            pass # directory already exists




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'about.ui'
#
# Created: Mon Jun 14 11:46:51 2010
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_About(object):
    def setupUi(self, About):
        About.setObjectName("About")
        About.resize(555, 475)
        self.buttonBox = QtGui.QDialogButtonBox(About)
        self.buttonBox.setGeometry(QtCore.QRect(360, 430, 181, 34))
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.lAbout = QtGui.QLabel(About)
        self.lAbout.setGeometry(QtCore.QRect(20, 20, 521, 391))
        self.lAbout.setObjectName("lAbout")

        self.retranslateUi(About)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("clicked(QAbstractButton*)"), About.close)
        QtCore.QMetaObject.connectSlotsByName(About)

    def retranslateUi(self, About):
        About.setWindowTitle(QtGui.QApplication.translate("About", "About Pynagram", None, QtGui.QApplication.UnicodeUTF8))
        self.lAbout.setText(QtGui.QApplication.translate("About", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:16pt; font-weight:600;\">Pynagram 1.0.0.61</span></p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:16pt; font-weight:600;\"></p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt;\">Copyright (c) 2009, 2010 Umang Varma. </span></p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://launchpad.net/pynagram\"><span style=\" text-decoration: underline; color:#0000ff;\">Pynagram on Launchpad</span></a></p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:11pt;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Pynagram is free software: you can redistribute it and/or modify</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">it under the terms of the GNU General Public License as published by</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">the Free Software Foundation, either version 3 of the License, or</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">(at your option) any later version.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Pynagram is distributed in the hope that it will be useful,</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">but WITHOUT ANY WARRANTY; without even the implied warranty of</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">GNU General Public License for more details.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">You should have received a copy of the GNU General Public License</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">along with Pynagram.  If not, see <a href=\"http://www.gnu.org/licenses/\"><span style=\" text-decoration: underline; color:#0000ff;\">http://www.gnu.org/licenses/</span></a>.</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'wordlists.ui'
#
# Created: Mon Jun  7 15:50:28 2010
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Wordlists(object):
    def setupUi(self, Wordlists):
        Wordlists.setObjectName("Wordlists")
        Wordlists.setWindowModality(QtCore.Qt.ApplicationModal)
        Wordlists.resize(535, 358)
        Wordlists.setModal(True)
        self.verticalLayoutWidget = QtGui.QWidget(Wordlists)
        self.verticalLayoutWidget.setGeometry(QtCore.QRect(10, 10, 511, 331))
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout = QtGui.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.wls = QtGui.QListWidget(self.verticalLayoutWidget)
        self.wls.setAcceptDrops(True)
        self.wls.setObjectName("wls")
        self.horizontalLayout.addWidget(self.wls)
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.pb_add = QtGui.QPushButton(self.verticalLayoutWidget)
        self.pb_add.setObjectName("pb_add")
        self.verticalLayout_2.addWidget(self.pb_add)
        self.pb_remove = QtGui.QPushButton(self.verticalLayoutWidget)
        self.pb_remove.setObjectName("pb_remove")
        self.verticalLayout_2.addWidget(self.pb_remove)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.pb_save = QtGui.QPushButton(self.verticalLayoutWidget)
        self.pb_save.setObjectName("pb_save")
        self.verticalLayout_2.addWidget(self.pb_save)
        self.pb_cancel = QtGui.QPushButton(self.verticalLayoutWidget)
        self.pb_cancel.setObjectName("pb_cancel")
        self.verticalLayout_2.addWidget(self.pb_cancel)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.label = QtGui.QLabel(self.verticalLayoutWidget)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)

        self.retranslateUi(Wordlists)
        QtCore.QMetaObject.connectSlotsByName(Wordlists)

    def retranslateUi(self, Wordlists):
        Wordlists.setWindowTitle(QtGui.QApplication.translate("Wordlists", "Word Lists", None, QtGui.QApplication.UnicodeUTF8))
        self.pb_add.setText(QtGui.QApplication.translate("Wordlists", "&Add", None, QtGui.QApplication.UnicodeUTF8))
        self.pb_remove.setText(QtGui.QApplication.translate("Wordlists", "&Remove", None, QtGui.QApplication.UnicodeUTF8))
        self.pb_save.setText(QtGui.QApplication.translate("Wordlists", "&Save", None, QtGui.QApplication.UnicodeUTF8))
        self.pb_cancel.setText(QtGui.QApplication.translate("Wordlists", "&Cancel", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Wordlists", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">You may drag and drop text from a <span style=\" font-style:italic;\">text-editor</span> or file(s) from a <span style=\" font-style:italic;\">file browser</span> onto the word list selector above to add word lists.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Click \'<span style=\" text-decoration: underline;\">S</span>ave\' to save your settings and start a new game.</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))




#!/usr/bin/env python
#
#     gui.py
#
#     Copyright (c) 2009, 2010 Umang Varma <umang.me@gmail.com>.
#
#     This file is part of Pynagram
#
#     Pynagram is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     Pynagram is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with Pynagram. If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4 import QtCore, QtGui
from qt_struct import Ui_MainWindow
from qt_about import Ui_About
from qt_wl import Ui_Wordlists

import random
import time


from pynagram.backend import anagram

class About(QtGui.QDialog):
    """This class is the About Dialog"""

    def __init__(self, parent=None, icon=None):
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_About()
        self.ui.setupUi(self)
        wind_icon = icon or QtGui.QIcon()
        self.setWindowIcon(wind_icon)

    def start(self):
        self.show()

class Wordlists(QtGui.QDialog, Ui_Wordlists):
    """This class is the Word lists Dialog"""

    def __init__(self, parent=None, icon=None, config=None):
        self.config = config
        QtGui.QWidget.__init__(self, parent)
        self.setupUi(self)
        wind_icon = icon or QtGui.QIcon()
        self.setWindowIcon(wind_icon)
        #
        QtCore.QObject.connect(self.pb_save, QtCore.SIGNAL("clicked()"), \
                                                self.saveconfig)
        QtCore.QObject.connect(self.pb_cancel, QtCore.SIGNAL("clicked()"), \
                                                self.cancelconfig)
        QtCore.QObject.connect(self.pb_add, QtCore.SIGNAL("clicked()"), \
                                                self.addwl)
        QtCore.QObject.connect(self.pb_remove, QtCore.SIGNAL("clicked()"), \
                                                self.removewl)
        #
        self.wls.__class__.dragEnterEvent = self.dragEnterEvent
        self.wls.__class__.dragMoveEvent = self.dragEnterEvent
        self.wls.__class__.dropEvent = self.drop
        self.readfromconfig()
       # 
    
    def addwl(self):
        files = list(QtGui.QFileDialog.getOpenFileNames(self, "Select word " + \
                "lists", "", "Text Files (*.txt);; All Files (*)"))
        for wl in files:
            self.config["wordlists"].append(str(wl))
        self.readfromconfig()

    def removewl(self):
        current_row = self.wls.currentRow()
        self.config["wordlists"].pop(current_row)
        self.readfromconfig()
        self.wls.setCurrentRow((self.wls.count() > current_row and current_row)
                or current_row -1)

    def saveconfig(self):
        self.config["wordlist"] = self.wls.currentRow()
        self.config.writeconfig()
        self.config.changed = True
        self.close()

    def cancelconfig(self):
        self.config.readconfig()
        self.close()

    def dragEnterEvent(self, event):
        if (event.mimeData().hasFormat("text/uri-list") or
                event.mimeData().hasFormat("text/plain")):
            event.accept()
        else:
            event.reject()

    def drop(self, event):
        data = event.mimeData()
        if data.hasFormat("text/uri-list"):
            self.config["wordlists"].extend( \
                    [str(x.toLocalFile()) for x in data.urls()])
            self.readfromconfig()
        elif data.hasFormat("text/plain"):
            path = "%s/wl%d.txt"%(self.config.pathtoconfig(), time.time())
            fsock = open(path , "w")
            fsock.write(data.text())
            fsock.close()
            self.config["wordlists"].append(path)
            self.readfromconfig()
    
    def readfromconfig(self):
        self.wls.clear()
        self.wls.addItems(self.config["wordlists"])
        self.wls.setCurrentRow(self.config["wordlist"])

class App(QtGui.QMainWindow):
    """This class handles the GUI."""

    def __init__(self, parent=None, config=None):
        """Initializes the code for the GUI"""
        self.pynagram = anagram.Anagram()
        self.config = config
        self.typed = []
        self.available_letters = []
        self.widgets = {}
        self.last_word = ""
        self.last_word_color = "000"
        self.solved = False
        self.time_started = 0
        self.time_elapsed = 0
        self.times_shuffled = 0
        self.timer = QtCore.QTimer()
        self.shuffle_timer = QtCore.QTimer()
        #
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.wind_icon = QtGui.QIcon()
        self.wind_icon.addFile('/usr/share/pixmaps/pynagram.xpm')
        self.wind_icon.addFile('icons/pixmaps/pynagram.xpm')
        self.setWindowIcon(self.wind_icon)
        #
        QtCore.QObject.connect(self.ui.actionSolve, \
                    QtCore.SIGNAL("activated()"), self.actionSolve_activate)
        QtCore.QObject.connect(self.ui.actionNew, \
                        QtCore.SIGNAL("activated()"), self.actionNew_activate)
        QtCore.QObject.connect(self.ui.actionAbout, \
                    QtCore.SIGNAL("activated()"), self.actionAbout_activate)
        QtCore.QObject.connect(self.ui.actionWordlists, \
                QtCore.SIGNAL("activated()"), self.actionWordlists_activate)
        QtCore.QObject.connect(self.timer, QtCore.SIGNAL("timeout()"), \
                                                    self.__update_status_bar)
        QtCore.QObject.connect(self.shuffle_timer, QtCore.SIGNAL("timeout()"),\
                                                                self.__shuffle)
        self.timer.start(1000)

    def init_game(self):
        """Initializes a game."""
        self.pynagram.start_new()
        self.available_letters = self.pynagram.letters[:]
        self.last_word = ""
        self.typed = []
        self.show()
        random.shuffle(self.available_letters)
        self.time_started = time.time()
        self.__reflect_letters()
        self.__update_words()
        self.times_shuffled = 0
        self.shuffle_timer.start(25)
        self.time_started = time.time()

    def __shuffle(self):
        """Shuffles the letters"""
        if self.times_shuffled < 10:
            random.shuffle(self.available_letters)
            self.resize(self.sizeHint())
            self.times_shuffled += 1
            self.__reflect_letters()
        else:
            self.shuffle_timer.stop()

    def __reflect_letters(self):
        """Updates the labels in the GUI

        to reflect the state of the variables in this class."""
        self.ui.l_avail.setText(" ".join(self.available_letters))
        self.ui.l_typed.setText(" ".join(self.typed))
        self.ui.l_last.setText(("Last word: <b><font color=\"#%s\">" + \
        "%s</font></b>") % (self.last_word_color, self.last_word))

    def __update_words(self):
        """Updates the list of words in the Pynagram window"""
        a_words = self.pynagram.words.keys()
        a_words.sort()
        a_words.sort(lambda x, y: len(x) - len(y))
        if not False in self.pynagram.words.values():
            self.solved = True
        per_column = len(a_words)/5 + 1
        for x in xrange(0, 5):
            words = [((self.pynagram.words[word] or self.solved) and  word) \
                or "_ " * len(word) for word in \
                        a_words[x*per_column:(x+1)*per_column]]
            getattr(self.ui, "l_solved_" + str(x+1)).setText(\
                "<br>".join([(word in self.pynagram.words and \
                self.pynagram.words[word] and self.solved and \
                (" <b>%s</b>"%word)) or word for \
                word in words]))
        self.__update_status_bar()

    def __update_status_bar(self):
        s_time = ""
        if not (self.solved or self.pynagram.solved_all):
            self.time_elapsed = int(time.time() - self.time_started)
        if self.time_elapsed >= 300 and not self.solved:
            self.solved = True
            self.actionSolve_activate()
        s_time = "Time: %s" % (time.strftime("%M:%S", \
                            time.gmtime(300 - self.time_elapsed)))
        s_score = "Score: %d" % self.pynagram.score
        self.ui.statusbar.showMessage(s_score + "\t" + s_time)

    def actionNew_activate(self):
        self.solved = False
        if not self.pynagram.qualified:
            self.pynagram.clear_all()
        self.init_game()

    def actionAbout_activate(self):
        w_about = About(icon=self.wind_icon)
        w_about.exec_()

    def actionWordlists_activate(self):
        self.config.changed = False
        w_wl = Wordlists(icon = self.wind_icon, config = self.config)
        w_wl.exec_()
        self.config.readconfig() # Reverts any unsaved config 
        if self.config.changed:
            self.config.changed = False
            wl = self.config["wordlists"][self.config.setdefault("wordlist", 0)]
            self.pynagram.read_from_file(wl)
            self.pynagram.qualified = False
            self.actionNew_activate()

    def actionSolve_activate(self):
        self.available_letters.extend(self.typed)
        self.typed = []
        self.__update_status_bar()
        self.solved = True
        self.__reflect_letters()
        self.__update_words()
        
    
    def keyPressEvent(self, event):
        if not self.solved:
            key = int(event.key())
            if 64 < key < 123 and chr(key).lower() in self.available_letters:
                # If an available letter has been typed
                letter = chr(key).lower()
                self.typed.append(letter)
                self.available_letters.remove(letter)
            elif key == 16777216:
                # Escape
                self.available_letters.extend(self.typed)
                self.typed = []
            elif key == 16777219 and len(self.typed) > 0:
                # Backspace
                self.available_letters.append(self.typed.pop())
            elif key == 32:
                # Spacebar
                random.shuffle(self.available_letters)
            elif key == 16777221 or key == 16777220:
                # Enter
                word = "".join(self.typed)
                result, already_typed = self.pynagram.guess(word)
                if already_typed:
                    self.last_word_color = "ff0"
                elif (not already_typed) and result:
                    self.last_word_color = "0f0"
                else:
                    self.last_word_color = "f00"
                self.last_word = word
                self.available_letters.extend(self.typed)
                self.typed = []
                self.__update_words()
            self.__reflect_letters()



#! /usr/bin/python
#
#     anagram.py
# 
#     Copyright (c) 2009, 2010 Umang Varma <umang.me@gmail.com>.
# 
#     This file is part of Pynagram
# 
#     Pynagram is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     Pynagram is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with Pynagram. If not, see <http://www.gnu.org/licenses/>.
# 

import random

class AnagramException(Exception): pass
class LengthException(AnagramException): pass
class WordException(AnagramException): pass


class Anagram:
    """The backend of pynagram that manages the current game.

    This class performs all the back-end tasks such as generating anagrams,
    checking if the input word exists, etc."""

    def __init__(self):
        """Initializes the class."""
        self.letters = []
        self.score = 0
        self.words = {}
        self.wordlist = []
        self.scoring = {3:3, 4:5, 5:8, 6:14, 7:20}
        self.solved_all = False
        self.qualified = False
        
    def read_from_file(self, wordlist):
        """Reads the wordlist from a file and stores it appropriately."""
        fsock = file(wordlist)
        lines = fsock.readlines()
        fsock.close()
        self.wordlist = self.filter_wordlist(lines)

    def filter_wordlist(self, lines):
        """Filters lines from a wordlist into a list."""
        return([x.strip() for x in lines if len(x) > 3])

    def find_letters(self, length=7):
        """Finds possible letters to make anagrams with from the wordlist"""
        # adapted from ActiveState Recipe #52560
        # ("Remove duplicates from a sequence")
        possible_words = [[word[x] for x in xrange(length)] \
            for word in self.wordlist if len(word) == length]
        #
        # Commented out the next few lines because the wordlist is not supposed
        # to have duplicates. It is none of our business to weed them out. If
        # any problems occur in the future, we'll uncomment these lines out.
        # These lines were initially supposed to remove duplicates in words.
        #
        # n = len(possible_words)
        # for x in xrange(n):
            # possible_words[x].sort()
        # possible_words.sort()
        # prev = possible_words[0]
        # next_free = 1
        # for x in xrange(1, n):
            # if possible_words[x] != prev:
                # possible_words[next_free] = prev = possible_words[x]
                # next_free += 1
        # possible_words = possible_words[:next_free]
        return random.choice(possible_words)

    def start_new(self, letters=None, length=7, no_word_error=False):
        """Starts a new game of Pynagram!"""
        self.qualified = False
        self.solved_all = False
        if (letters is not None) and len(letters) == 7:
            self.letters = letters
        elif (letters is not None) and len(letters) >= 3:
            raise LengthException("Bad number of letters given.")
        else:
            self.letters = self.find_letters(length)
        words = []
        for word in self.wordlist:
            is_anagram = True
            a_letters = self.letters[:]
            for x in word:
                if x in a_letters:
                    a_letters.remove(x)
                else:
                    is_anagram = False
                    break
            if is_anagram:
                words.append(word)
        words.sort(lambda x, y: len(y) - len(x))
        if len(words[0]) < length:
            raise WordException("There are no %d letter words that match \
this anagram" % length)
        self.words = dict([(word, False) for word in words])

    def guess(self, word):
        """Attempts to solve (guess) an anagram"""
        if word in self.words:
            typed = self.words[word]
            self.words[word] = True
            if not typed:
                self.score += self.scoring[len(word)]
                if not self.qualified and len(word) is 7:
                    self.score += 30
                    self.qualified = True
                if not False in self.words.values() and not self.solved_all:
                    self.solved_all = True
                    self.score += 30
            return (True, typed)
        else:
            return (False, False)

    def clear_all(self):
        """Clears all data being used for a new game."""
        self.score = 0
        self.letters = []
        self.words = {}
        self.qualified = False



#! /usr/bin/python
# 
#     setup.py
# 
#     Copyright (c) 2009, 2010 Umang Varma <umang.me@gmail.com>.
# 
#     This file is part of Pynagram
# 
#     Pynagram is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     Pynagram is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with Pynagram. If not, see <http://www.gnu.org/licenses/>.
#
from distutils.core import setup
import glob
#import py2exe

r = setup(name= "pynagram",
        version= "1.0.0",
        description= "Pynagram - Unjumble the letters and get addicted!",
        author= "Umang Varma",
        author_email= "umang.me@gmail.com",
        url= "https://launchpad.net/pynagram",
        packages= ["pynagram", "pynagram.gui", "pynagram.backend"],
        scripts= ["bin/pynagram"],
        data_files= [("share/pynagram", glob.glob("wordlist/*"))],
        windows= [{"script": "bin/pynagram", "icon_resources": [(1, "icons/pynagram.ico")]}],
        options={"py2exe": {"skip_archive": False, "includes": ["sip"]}},
        license="GPLv3",
        long_description="""anagram word game
Pynagram is a simple word game in which the player is presented with seven
letters in a random order. Every word that is "solved" from the jumbled letters
earns the player points. If the player solves one or more seven letter words,
the score is carried over into the next game.

Pynagram was inspired from Anagramarama."""
        )
